package main

import (
	"fmt"
	"math/rand"
	"time"

	"example.com/matrix"
)

func main() {

	var ret int16
	var ret32 int32

	matrix.Init_MatrixAPI()

	ret32 = matrix.GetVersionAPI()
	fmt.Println("VersionAPI=", ret32>>16&0xffff, ".", ret32&0xffff)
	ret32 = matrix.GetVersionDRV_USB()
	fmt.Println("VersionDRVUSB=", ret32>>16&0xffff, ".", ret32&0xffff)

	ret = matrix.Dongle_Count()
	fmt.Println("count=", ret)

	if ret < 1 {
		fmt.Println("Connect a dongle")
		return
	}

	var UserCode int32 = 1234
	var FieldData []int32

	if ret > 0 {

		// ***** dongle info **********
		fmt.Println("*** Info on the 1st dongle ***")

		ret32 = matrix.Dongle_Version(1)
		fmt.Println("Dongle Version=", ret32>>16&0xffff, ".", ret32&0xffff)

		fmt.Println("Dongle Model=", matrix.Dongle_Model(1))
		fmt.Println("Dongle MemSize=", matrix.Dongle_MemSize(1))

		serNr := matrix.Dongle_ReadSerNr(UserCode, 1)
		fmt.Println("SerNr=", serNr)

		// ***** read/write **********
		fmt.Println("")
		fmt.Println("*** Read/Write Test ***")

		FieldData = make([]int32, 15)

		rand.Seed(time.Now().UnixNano())
		for i := 0; i < 15; i++ {
			// field 5 will be used as AppSlot later
			if i == 4 {
				FieldData[i] = 10
			} else {
				FieldData[i] = int32(rand.Uint32())
			}
		}

		ret = matrix.Dongle_WriteDataEx(UserCode, FieldData, 1, 10, 1)
		fmt.Println("Dongle_WriteDataEx returned ", ret)

		FieldData = nil
		FieldData = make([]int32, 15)
		ret = matrix.Dongle_ReadData(UserCode, FieldData, 15, 1)
		fmt.Println("Dongle_ReadData returned ", ret)

		for idx, data := range FieldData {
			fmt.Println("idx=", idx, " Data=", uint32(data))
		}

		FieldData = nil
		FieldData = make([]int32, 10)
		ret = matrix.Dongle_ReadDataEx(UserCode, FieldData, 5, 10, 1)
		fmt.Println("Dongle_ReadDataEx returned ", ret)

		for idx, data := range FieldData {
			fmt.Println("idx=", idx, "Data=", uint32(data))
		}

		// ***** software/hardware encryption and decryption **********
		fmt.Println("")
		fmt.Println("*** Encryption/Decryption Test ***")

		// generate a key( uint32)
		var Key [4]uint32
		rand.Seed(time.Now().UnixNano())
		for i := 0; i < 4; i++ {
			Key[i] = rand.Uint32()
		}

		// copy uint32 key to int32 key
		Key1 := [4]int32{int32(Key[0]), int32(Key[1]), int32(Key[2]), int32(Key[3])}

		// write the key to hardware
		ret = matrix.Dongle_WriteKey(UserCode, Key1, 1)
		fmt.Println("Dongle_WriteKey returned ", ret)

		// genreate random data
		var Data [2]uint32
		rand.Seed(time.Now().UnixNano())
		Data[0] = rand.Uint32()
		Data[1] = rand.Uint32()

		Data1 := []int32{int32(Data[0]), int32(Data[1])}

		// software encryption
		matrix.MxApp_Encrypt(&Data, Key)
		fmt.Println("Encrypted Data=", Data)

		// hardware encryption
		ret = matrix.Dongle_EncryptData(UserCode, Data1, 1)
		fmt.Println("Dongle_EncryptData returned ", ret)
		fmt.Println("Encrypted Data1=", uint32(Data1[0]), uint32(Data1[1]))

		// software/hardeware encryption should produce the same results
		if Data[0] != uint32(Data1[0]) || Data[1] != uint32(Data1[1]) {
			fmt.Println("*********************************")
			fmt.Println("Encryption Mismatch!!!")
			fmt.Println("*********************************")
		}

		//software decryption
		matrix.MxApp_Decrypt(&Data, Key)
		fmt.Println("DecryptedData=", Data)

		//hardware decryption
		ret = matrix.Dongle_DecryptData(UserCode, Data1, 1)
		fmt.Println("Dongle_DecryptData returned ", ret)
		fmt.Println("Decrypted Data1=", uint32(Data1[0]), uint32(Data1[1]))

		// software/hardeware decryption should produce the same results
		if Data[0] != uint32(Data1[0]) || Data[1] != uint32(Data1[1]) {
			fmt.Println("*********************************")
			fmt.Println("Decryption Mismatch!!!")
			fmt.Println("*********************************")
		}

		// ***** Network API test **********
		fmt.Println("")
		fmt.Println("*** Network API Test ***")

		ret = matrix.SetConfig_MatrixNet(1, "\\\\server\\shared\\serverfile")
		fmt.Println("SetConfig_MatrixNet returned ", ret)

		ret = matrix.LogIn_MatrixNet(UserCode, 5, 1)
		fmt.Println("LogIn_MatrixNet returned ", ret)

		ret = matrix.Dongle_Count()
		fmt.Println("  server dongle count=", ret)

		if ret > 0 {
			serNr := matrix.Dongle_ReadSerNr(UserCode, 1)
			fmt.Println("  server dongle SerNr=", serNr)
		}

		ret = matrix.LogOut_MatrixNet(UserCode, 5, 1)
		fmt.Println("LogOut_MatrixNet returned ", ret)

	}
	matrix.Release_MatrixAPI()

}
