package matrix

// We assume you are running GO on 64bit Windows and have installed 64bit version of MinGW
//
// How to install 64bit MinGW
// https://qiita.com/Marny/items/133e7032c3d20d2e77b1
//
// 1. Install Matrix SDK to c:\matrix
// 2. For this module, we copied the following 2 libs to C:/matrix/API/lib/msvc folder
//    as bufferoverflowu64.lib and winspool64.lib respectively
// - "C:\Program Files (x86)\Windows Kits\10\Lib\10.0.19041.0\um\x64\BufferOverflowU.lib"
// - "C:\Program Files (x86)\Windows Kits\10\Lib\10.0.19041.0\um\x64\WinSpool.Lib"
//
//  Building this module, you get several lines of "Warning: corrupt .drectve at end of def file"
//  but the built program should work

//
// The header file matrix32.h includes mxtypes.h in the same directory.
// Be sure that _mxINT32 macro expands to int (32bit), not to long (64bit) in mxtypes.h
// The default setting expands it to long
//
//   #define  _mxINT32  int
//   #define  _mxUINT32 unsigned int
//

/*
#cgo LDFLAGS: -L C:/matrix/API/lib/msvc   -l mxst64_vc100_mt -l bufferoverflowu64 -l winspool64
#include <stdlib.h>
#include "c:\matrix\API\lib\h\matrix32.h"
void __fastcall __GSHandlerCheck() {}
void __fastcall __chkstk() {}

double __imp__difftime64( __time64_t __t1, __time64_t __t0 )
{
  return (double)((long double)(__t1) - (long double)(__t0));
}

*/
import (
	"C"
)

import "unsafe"

func Init_MatrixAPI() (ret int16) {

	ret = int16(C.Init_MatrixAPI())
	return
}

func Release_MatrixAPI() (ret int16) {

	ret = int16(C.Release_MatrixAPI())
	return
}

func GetVersionAPI() (ret int32) {
	ret = int32(C.GetVersionAPI())
	return
}

func GetVersionDRV() (ret int32) {
	ret = int32(C.GetVersionDRV())
	return
}

func GetVersionDRV_USB() (ret int32) {
	ret = int32(C.GetVersionDRV_USB())
	return
}

func Dongle_Version(DngNr int16) (ret int32) {
	ret = int32(C.Dongle_Version(C.short(DngNr), 85))
	return
}

func Dongle_Model(DngNr int16) (ret int32) {
	ret = int32(C.Dongle_Model(C.short(DngNr), 85))
	return
}

func Dongle_MemSize(DngNr int16) (ret int16) {
	ret = int16(C.Dongle_MemSize(C.short(DngNr), 85))
	return
}

func Dongle_Count() (ret int16) {

	ret = int16(C.Dongle_Count(85))
	return
}

func Dongle_ReadSerNr(UserCode int32, DngNr int16) (ret int32) {

	ret = int32(C.Dongle_ReadSerNr(C.int(UserCode), C.short(DngNr), 85))
	return
}

func Dongle_ReadData(UserCode int32, Data []int32, Count int16, DngNr int16) (ret int16) {

	ret = int16(C.Dongle_ReadData(C.int(UserCode), (*C.int)(&Data[0]), C.short(Count), C.short(DngNr), 85))
	return
}

func Dongle_ReadDataEx(UserCode int32, Data []int32, Fpos int16, Count int16, DngNr int16) (ret int16) {

	ret = int16(C.Dongle_ReadDataEx(C.int(UserCode), (*C.int)(&Data[0]), C.short(Fpos), C.short(Count), C.short(DngNr), 85))
	return
}

func Dongle_WriteData(UserCode int32, Data []int32, Count int16, DngNr int16) (ret int16) {

	ret = int16(C.Dongle_WriteData(C.int(UserCode), (*C.int)(&Data[0]), C.short(Count), C.short(DngNr), 85))
	return
}

func Dongle_WriteDataEx(UserCode int32, Data []int32, Fpos int16, Count int16, DngNr int16) (ret int16) {

	ret = int16(C.Dongle_WriteDataEx(C.int(UserCode), (*C.int)(&Data[0]), C.short(Fpos), C.short(Count), C.short(DngNr), 85))
	return
}

func Dongle_WriteKey(UserCode int32, KeyData [4]int32, DngNr int16) (ret int16) {
	ret = int16(C.Dongle_WriteKey(C.int(UserCode), (*C.int)(&KeyData[0]), C.short(DngNr), 85))
	return
}

func Dongle_GetKeyFlag(UserCode int32, DngNr int16) (ret int16) {
	ret = int16(C.Dongle_GetKeyFlag(C.int(UserCode), C.short(DngNr), 85))
	return
}

func Dongle_EncryptData(UserCode int32, DataBlock []int32, DngNr int16) (ret int16) {
	ret = int16(C.Dongle_EncryptData(C.int(UserCode), (*C.int)(&DataBlock[0]), C.short(DngNr), 85))
	return
}

func Dongle_DecryptData(UserCode int32, DataBlock []int32, DngNr int16) (ret int16) {
	ret = int16(C.Dongle_DecryptData(C.int(UserCode), (*C.int)(&DataBlock[0]), C.short(DngNr), 85))
	return
}

func Dongle_SetDriverFlag(UserCode int32, DriverFlag int16, DngNr int16) (ret int16) {
	ret = int16(C.Dongle_SetDriverFlag(C.int(UserCode), C.short(DriverFlag), C.short(DngNr), 85))
	return
}

func Dongle_GetDriverFlag(UserCode int32, DngNr int16) (ret int16) {
	ret = int16(C.Dongle_GetDriverFlag(C.int(UserCode), C.short(DngNr), 85))
	return
}

func SetConfig_MatrixNet(nAccess int16, nFile string) (ret int16) {
	serverFile := C.CString(nFile)
	ret = int16(C.SetConfig_MatrixNet(C.short(nAccess), serverFile))
	C.free(unsafe.Pointer(serverFile))
	return
}

func GetConfig_MatrixNet(Category int16) (ret int32) {
	ret = int32(C.GetConfig_MatrixNet(C.short(Category)))
	return
}

func LogIn_MatrixNet(UserCode int32, AppSlot int16, DngNr int16) (ret int16) {
	ret = int16(C.LogIn_MatrixNet(C.int(UserCode), C.short(AppSlot), C.short(DngNr)))
	return
}

func LogOut_MatrixNet(UserCode int32, AppSlot int16, DngNr int16) (ret int16) {
	ret = int16(C.LogOut_MatrixNet(C.int(UserCode), C.short(AppSlot), C.short(DngNr)))
	return
}

func MxApp_Encrypt(Data *[2]uint32, Key [4]uint32) int16 {
	var delta, sum uint32
	var cnt int16

	sum = 0
	delta = 0x9E3779B9
	cnt = 32
	for cnt = 32; cnt > 0; cnt-- {
		Data[0] += ((Data[1]<<4 ^ Data[1]>>5) + Data[1]) ^ (sum + Key[sum&3])
		sum += delta
		Data[1] += ((Data[0]<<4 ^ Data[0]>>5) + Data[0]) ^ (sum + Key[sum>>11&3])
	}
	return 0
}

func MxApp_Decrypt(Data *[2]uint32, Key [4]uint32) int16 {
	var delta, sum uint32
	var cnt int16

	sum = 0xC6EF3720
	delta = 0x9E3779B9
	cnt = 32
	for cnt = 32; cnt > 0; cnt-- {
		Data[1] -= ((Data[0]<<4 ^ Data[0]>>5) + Data[0]) ^ (sum + Key[sum>>11&3])
		sum -= delta
		Data[0] -= ((Data[1]<<4 ^ Data[1]>>5) + Data[1]) ^ (sum + Key[sum&3])
	}
	return 0
}
